# Mikel Broström 🔥 BoxMOT 🧾 AGPL-3.0 license

import cv2
import numpy as np
import torch
from PIL import Image
from transformers import RTDetrImageProcessor, RTDetrV2ForObjectDetection
from ultralytics.engine.results import Results
from ultralytics.models.yolo.detect import DetectionPredictor

from boxmot.utils import logger as LOGGER


class RTDetrStrategy:
    pt = False
    stride = 32
    fp16 = False
    triton = False
    ch = 3

    def __init__(self, model, device, args):
        self.args = args
        self.device = device

        model = str(model)
        if model.endswith(".pt"):
            model = model[:-3]
        if not model.startswith("PekingU/"):
            model = f"PekingU/{model}"

        LOGGER.info(f"Loading RTDetr model: {model}")
        
        # Load model and processor from Hugging Face
        self.image_processor = RTDetrImageProcessor.from_pretrained(model)
        self.model = RTDetrV2ForObjectDetection.from_pretrained(model).to(device)

        # Get class names from model config
        self.names = self.model.config.id2label

    @torch.no_grad()
    def __call__(self, im, augment, visualize, embed):
        if isinstance(im, torch.Tensor):
            im = im.cpu().numpy()
            if im.ndim == 3 and im.shape[0] == 3:
                im = im.transpose(1, 2, 0)
            im = np.ascontiguousarray(im)

        # Convert numpy image (BGR) to PIL Image (RGB)
        image = Image.fromarray(cv2.cvtColor(im, cv2.COLOR_BGR2RGB))
        
        inputs = self.image_processor(images=image, return_tensors="pt").to(self.device)
        outputs = self.model(**inputs)
        
        results = self.image_processor.post_process_object_detection(
            outputs, 
            target_sizes=torch.tensor([(image.height, image.width)], device=self.device), 
            threshold=self.args.conf
        )
        
        # Format results: [x1, y1, x2, y2, conf, cls]
        detections = []
        for result in results:
            for score, label, box in zip(result["scores"], result["labels"], result["boxes"]):
                box = box.cpu().numpy()
                score = score.item()
                label = label.item()
                detections.append([*box, score, label])
                
        if not detections:
            return torch.zeros((1, 0, 6), device=self.device)
            
        return torch.tensor(detections, device=self.device).unsqueeze(0)

    def warmup(self, imgsz):
        pass

    def update_im_paths(self, predictor: DetectionPredictor):
        """
        This function saves image paths for the current batch,
        being passed as callback on_predict_batch_start
        """
        assert isinstance(
            predictor, DetectionPredictor
        ), "Only ultralytics predictors are supported"
        self.im_paths = predictor.batch[0]

    def preprocess(self, im) -> torch.Tensor:
        # RTDetr expects PIL images or list of them, but here we just pass through
        # The actual preprocessing happens in __call__
        assert isinstance(im, list)
        return im[0]

    def postprocess(self, preds, im, im0s):
        results = []
        for i, pred in enumerate(preds):
            im_path = self.im_paths[i] if hasattr(self, 'im_paths') and len(self.im_paths) else ""

            if pred is None or len(pred) == 0:
                pred = torch.empty((0, 6), device=self.device)
                results.append(
                    Results(path=im_path, boxes=pred, orig_img=im0s[i], names=self.names)
                )
                continue
            
            if self.args.classes:
                pred = pred[
                    torch.isin(pred[:, 5].cpu(), torch.as_tensor(self.args.classes))
                ]
            
            results.append(
                Results(path=im_path, boxes=pred, orig_img=im0s[i], names=self.names)
            )
        return results



# Mikel Broström 🔥 BoxMOT 🧾 AGPL-3.0 license

from boxmot.utils import logger as LOGGER
from boxmot.utils.checks import RequirementsChecker

checker = RequirementsChecker()

ULTRALYTICS_MODELS = {"yolov8", "yolov9", "yolov10", "yolo11", "yolo12", "sam"}
RTDETR_MODELS = {"rtdetr_v2_r50vd", "rtdetr_v2_r18vd", "rtdetr_v2_r101vd"}
YOLOX_MODELS = {"yolox_n", "yolox_s", "yolox_m", "yolox_l", "yolox_x"}


def _check_model(name, markers):
    """Check if model name contains any of the markers."""
    return any(m in str(name) for m in markers)


def is_ultralytics_model(yolo_name):
    return _check_model(yolo_name, ULTRALYTICS_MODELS)


def is_yolox_model(yolo_name):
    return _check_model(yolo_name, YOLOX_MODELS)


def is_rtdetr_model(yolo_name):
    return _check_model(yolo_name, RTDETR_MODELS)


def default_imgsz(yolo_name):
    if is_ultralytics_model(yolo_name):
        return [640, 640]
    elif is_yolox_model(yolo_name):
        return [800, 1440]
    else:
        return [640, 640]


def get_yolo_inferer(yolo_model):
    """
    Determines and returns the appropriate inference strategy class based on the model name.
    Handles dependency checks and imports dynamically.
    """
    model_name = str(yolo_model)

    strategies = [
        (
            is_yolox_model,
            ("yolox", "tabulate", "thop"),
            {"yolox": ["--no-deps"]},
            "boxmot.detectors.yolox",
            "YoloXStrategy",
        ),
        (
            is_ultralytics_model,
            (),
            {},
            "boxmot.detectors.ultralytics",
            "UltralyticsStrategy",
        ),
        (
            is_rtdetr_model,
            ("transformers[torch]", "timm"),
            {},
            "boxmot.detectors.rtdetr",
            "RTDetrStrategy",
        ),
    ]

    for check_func, packages, extra_args, module_path, class_name in strategies:
        if check_func(model_name):
            for package in packages:
                try:
                    # Simple import check for package name (stripping version/extras)
                    pkg_name = package.split("[")[0].split("=")[0]
                    __import__(pkg_name)
                except ImportError:
                    args = extra_args.get(pkg_name, [])
                    checker.check_packages((package,), extra_args=args)

            module = __import__(module_path, fromlist=[class_name])
            return getattr(module, class_name)

    LOGGER.error(f"Failed to infer inference mode from yolo model name: {model_name}")
    LOGGER.error("Supported models must contain one of the following:")
    LOGGER.error(f"  Ultralytics: {ULTRALYTICS_MODELS}")
    LOGGER.error(f"  RTDetr: {RTDETR_MODELS}")
    LOGGER.error(f"  YOLOX: {YOLOX_MODELS}")
    LOGGER.error(
        "By using these names, the default COCO-trained models will be downloaded automatically. "
        "For custom models, the filename must include one of these substrings to route it to the correct package and architecture."
    )
    exit()

